/* eslint-disable */
'use strict'
import LansaContext from './lansa-context'
import axios from 'axios'
import defaultConfig from './default-config'
import * as Utils from './utils'



class LansaRequest {
  
    constructor(ownerPlugin, reqOptions) {
        this.Owner = ownerPlugin
        this.requestType = reqOptions.requestType
        this.ModuleName = reqOptions.ModuleName
        this.RoutineName = reqOptions.RoutineName
        
        this.requestdata = { 
            context: {},
            fields: {},
            lists: {}
        }
        
        this.responsedata = {
            status: Utils.MakeVoidStatus(),
            webroutine: null
        }
    }


    SetSessionKey(strSessionKey) {

        if (this.requestdata.context === null)
        {
            this.requestdata.context = Utils.FindLansaContext() || LansaContext
        }

        this.requestdata.context["session-key"] = strSessionKey
        this.requestdata.context["session-key-method"] = "hidden"
        this.requestdata.context["session-key-name"] = "_SESSIONKEY"

    }


    AddField(fieldName, fieldValue) {
        if (this.requestdata.fields[fieldName] == undefined)
        {
            this.requestdata.fields[fieldName] = { value: "" }
        }
        this.requestdata.fields[fieldName].value = fieldValue
    }

    DefineList(listname, listfields) {
        this.requestdata.lists[listname] = {
            headers: [],
            entries: []
        }

        let idx = 0
        for (let c in listfields) 
        {
            let nwc = {
                'name': listfields[c]
            }
            this.requestdata.lists[listname].headers.push(nwc)
            idx++
        }
    }

    AddEntryToList(listname, record) {
        if (this.requestdata.lists[listname] == undefined)
            return;
        
        this.requestdata.lists[listname].entries.push(record)
    }
   

    GetStatus() {
        return {
            code: this.responsedata.status.code,
            message: this.responsedata.status.messageid + " - " + this.responsedata.status.message
        }
    }

    GetFieldValue(fieldname) {
        return (this.responsedata.webroutine.fields[fieldname] && this.responsedata.webroutine.fields[fieldname].value) || ""
    }
    
    GetList(listName) {        
        if (this.responsedata.webroutine.lists && this.responsedata.webroutine.lists[listName]) {
            
            let pl = this.responsedata.webroutine.lists[listName]
            
            let retList = {
                "listname": listName,
                "columns" : [],
                "entries" : []
            }

            for (let c = 0; c < pl.header.length; c++)
            {
                retList.columns.push(
                    {
                        "colname" : pl.header[c].name
                    }
                )
            }        

            for  (let r = 0; r < pl.entries.length; r++)
            {
                let rec = {
                    "cells" : {}
                }

                for (let c = 0; c < pl.header.length; c++)
                {
                    rec.cells[pl.header[c].name] = {"value": pl.entries[r][c]}
                }

                retList.entries.push(rec)
            }

            return retList
        }
    }   

    LogMessage(argMessage,argWr,argData) {

        if (this.Owner.enableLog) console.log(argWr + " ( " + argData + " ): " + argMessage);

    }

    Execute() {

        // ===============================================================
        // PREPARAZIONE RICHIESTA AXIOS
        // ===============================================================

        let requestdata = null
        let querystringseparator = (this.Owner.lansaVersion < 12) ? "+" : "&"

        if (this.requestType == "MODULE") {

            if (this.requestdata.context === null) {

                this.requestdata.context = Utils.FindLansaContext() || LansaContext

            }

            requestdata.webroutine = this.requestdata;
        
        } else {

            requestdata = Utils.MakeFormEncodedBody(this.requestdata);

        }
        
        let requesturl = this.Owner.urlPrefix + '/cgi-bin/lansaweb?';
        requesturl += 'w=' + this.ModuleName;
        requesturl += querystringseparator + 'r=' + this.RoutineName;
        requesturl += querystringseparator + (this.requestType === "MODULE" ? 'vlweb=1' : 'ml=LANSA:XHTML');
        requesturl += querystringseparator + 'part=' + this.Owner.partition;
        requesturl += querystringseparator + 'lang=' + "ITL";
                
        let cfg = {
            ...defaultConfig,
            }

        if (this.requestType == "WAM") {

            cfg.headers = {
                ...cfg.headers,
            }

            cfg.wam = this.ModuleName;
            cfg.wr = this.RoutineName;
            cfg.reqid = String.fromCharCode(65 + Math.floor(Math.random() * 26)) + Date.now().toString();

        }

        // ===============================================================
        // ESECUZIONE RICHIESTA AXIOS
        // ===============================================================

        return axios.post( requesturl , requestdata , cfg ).then( response => {          

            // CONTROLLO ERRORI

            let isError = false;

            if ((this.requestType === "WAM") && (typeof response.data === "string")) {

                if (response.data.includes('LANSA for the Web Error')) isError = true;

            }

            if (this.requestType === "MODULE") {

                if (response.data["vlweb-error"]) isError = true;

            }

            if (isError) {
                
                let error = new Error();

                error.response = response;
                error.isLansaError = true;

                return Promise.reject(error)

            } else {

                let WebRuotineResponse = null

                if ( (this.requestType === "MODULE") && (response.data.webroutine) ) {
    
                    WebRuotineResponse = response.data.webroutine
                
                }
    
                if ( (this.requestType === "WAM") && (response.data) ) {
    
                    WebRuotineResponse = response.data
    
                }
    
                this.responsedata = {
                    status: Utils.MakeGoodStatus(),
                    webroutine: WebRuotineResponse
                }               
                
                if ( (this.requestType === "MODULE") && ( this.Owner.associatedStore ) )
                {
                    for (let fldname in this.responsedata.webroutine.fields) {
                        let mutationName = 'SET_' + fldname
                        if (this.Owner.associatedStore._mutations[mutationName]) {
                            this.Owner.associatedStore.commit(mutationName, this.responsedata.webroutine.fields[fldname].value)
                        }
                    }
                }

                return Promise.resolve();
            
            }

        }).catch( error => {
           
            let respstatus = {}

            if (error.isLansaError) {

                if (this.requestType === "MODULE") {

                    respstatus = Utils.MakeBadStatus("ER", error.response.data["vlweb-error"]["message"], error.response.data["vlweb-error"]["message-id"]);

                } else {

                    respstatus = Utils.MakeBadStatus("ER", "Unrecoverable error occurred while executing the Web application routine");

                }

                this.Owner.onResponseErrorFunction(error); // FUORI INTERCEPTORS IN QUANTO ERRORE GENERATO MANUALMENTE

            } else {

                if (error.request) {

                    respstatus = Utils.MakeBadStatus("E1", "Error occurred during request performing")
    
                } else {
    
                    respstatus = Utils.MakeBadStatus("E2", error.message)
    
                }

            }

            this.responsedata = {
                status: respstatus,
                context: this.requestdata.context,
                fields: {},
                lists: {}
            }

            return Promise.reject(respstatus.message);

        })
        
    }

}

export default LansaRequest