/* eslint-disable */
'use strict'

import axios from 'axios'
import LansaRequest from './lansa-request'
import * as Tools from './utils'

/* ======================================================================= *\
    LANSA OPTIONS:

    lansaVersion:        [11.5 / 12 / 13 / 14]
    urlPrefix:           base Url for request
    associatedStore:     associated Vuex Store    
    partition:           override partition to use

\* ======================================================================= */


class LansaVue {

    constructor(lansaoptions) {
        
        this.lansaVersion = parseInt((lansaoptions && lansaoptions.lansaVersion) || "14")
        this.urlPrefix = (lansaoptions && lansaoptions.urlPrefix) || ""
        this.associatedStore = (lansaoptions && lansaoptions.associatedStore) || null
        this.partition = (lansaoptions && lansaoptions.partition) || 'CAL'
        this.enableLog = (lansaoptions && lansaoptions.enableLog) || false
        this.onRequestFunction = (lansaoptions && lansaoptions.onRequestFunction) || null
        this.onRequestErrorFunction = (lansaoptions && lansaoptions.onRequestErrorFunction) || null
        this.onResponseFunction = (lansaoptions && lansaoptions.onResponseFunction) || null
        this.onResponseErrorFunction = (lansaoptions && lansaoptions.onResponseErrorFunction) || null

        if (this.onRequestFunction || this.onRequestErrorFunction) {

            axios.interceptors.request.use( (config) => {

                if (this.onRequestFunction) this.onRequestFunction(config);

                return config;

            }, (error) => {
        
                if (this.onRequestErrorFunction) this.onRequestErrorFunction(error);

                return Promise.reject(error);

            })

        }

        if (this.onResponseFunction || this.onResponseErrorFunction) {

            axios.interceptors.response.use( (response) => {

                if (this.onResponseFunction) this.onResponseFunction(response);

                return response;

            }, (error) => {

                if (this.onResponseErrorFunction) this.onResponseErrorFunction(error);
        
                return Promise.reject(error);

            })

        }

    }

    InitializeLansaContext() {
        if (this.lansaVersion < 14) {
            return null
        
        } else {
            
            return Tools.FindLansaContext()
        }

    }

    MakeNewModuleRequest(ServerModule, ServerRoutine) {
        return new LansaRequest(this, {
            requestType: "MODULE",
            ModuleName: ServerModule,
            RoutineName: ServerRoutine
        })
    }

    MakeNewWAMRequest(WamName, WebRoutine) {
        return new LansaRequest(this, {
            requestType: "WAM",
            ModuleName: WamName,
            RoutineName: WebRoutine
        })
    }    
}

export default LansaVue